package report

import (
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/models/ht"
	productDao "eta/eta_mini_ht_api/models/merchant"
	reportDao "eta/eta_mini_ht_api/models/report"
	"fmt"
	"strconv"
	"strings"
)

func GetHTReportIdsByPermissionIds(permissionIds []int) (ids []int, err error) {
	//品种名字筛选
	permissionNames, err := GetPermissionNamesByPermissionIds(permissionIds)
	if err != nil {
		logger.Error("获取品种名称列表失败:%v", err)
		return
	}
	//更具query查询ids TODO
	ids, err = reportDao.GetIdsByPlateNames(permissionNames)
	return
}
func GetHTReportIdsByPermissionIdsWithRiskLevel(permissionIds []int, riskLevel string) (orgIds []int, err error) {
	//品种名字筛选
	var permissionNames []string
	permissionNames, err = GetPermissionNamesByPermissionIds(permissionIds)
	if err != nil {
		logger.Error("获取品种名称列表失败:%v", err)
		return
	}
	//更具query查询ids
	var reports []reportDao.Report
	reports, err = reportDao.GetReportByPlateNames(permissionNames)
	if err != nil {
		logger.Error("获取HT研报列表失败:%v", err)
		return
	}
	if riskLevel == "" {
		for _, report := range reports {
			orgIds = append(orgIds, report.OrgID)
		}
		return
	}
	//根据riskLevel过滤ids
	var ids []int
	for _, report := range reports {
		ids = append(ids, report.ID)
	}
	var productList []productDao.MerchantProduct
	//现加入没有产品的报告
	productList, err = productDao.GetProductListBySourceIds(ids, productDao.Report)
	for _, report := range reports {
		find := false
		for _, product := range productList {
			if product.SourceID == report.ID {
				find = true
			}
		}
		if !find {
			orgIds = append(orgIds, report.OrgID)
		}
	}
	riskNum, err := parseRiskLevel(riskLevel)
	if err != nil {
		logger.Error("解析风险等级失败:%v", err)
		return
	}
	//再把符合风险等级的产品报告加入
	//productList, err = productDao.GetProductListBySourceIds(ids, productDao.Report)
	for _, report := range reports {
		for _, product := range productList {
			if product.SourceID == report.ID {
				pdRiskNum, pdErr := parseRiskLevel(product.RiskLevel)
				if pdErr != nil {
					logger.Error("解析产品风险等级失败:%v,产品id:%v", pdErr, product.ID)
					continue
				}
				if pdRiskNum <= riskNum {
					orgIds = append(orgIds, report.OrgID)
				}
			}
		}
	}
	return
}
func GetHtReport(id int) (pdf string, err error) {
	return ht.GetPDFUrl(id)
}

func parseRiskLevel(level string) (int, error) {
	parts := strings.Split(level, "R")
	if len(parts) < 2 {
		return 0, fmt.Errorf("无效的风险等级: %s", level)
	}
	numberStr := parts[1]
	number, err := strconv.Atoi(numberStr)
	if err != nil {
		return 0, fmt.Errorf("无法将风险等级转换为数字: %s", err)
	}
	return number, nil
}
