package user

import (
	"errors"
	logger "eta/eta_mini_ht_api/common/component/log"
	userDao "eta/eta_mini_ht_api/models/user"
	"fmt"
	"time"
)

type RecordCountDTO struct {
	UserId     int
	TraceId    string
	Mobile     string
	SourceId   int
	SourceType userDao.SourceType
	IpAddress  string
	Location   string
	Referer    string
	Additional string
}

const (
	video = "video"
	audio = "audio"
)

func getSourceType(mediaType string) userDao.SourceType {
	switch mediaType {
	case video:
		return userDao.VideoSourceType
	case audio:
		return userDao.AudioSourceType
	default:
		return ""
	}
}
func generateTraceId(productType string) string {
	return fmt.Sprintf("%s%d", productType, time.Now().UnixNano())
}
func CountReport(record RecordCountDTO) (traceId string, err error) {
	dao := convertUserToReportFlow(record)
	if dao.TraceId == "" {
		traceId = generateTraceId(string(dao.SourceType))
		dao.TraceId = traceId
		err = userDao.CountSourceClicks(dao)
	} else {
		var dbRecord userDao.UserSourceClickFlow
		dbRecord, err = userDao.GetClickRecordByTraceId(dao.TraceId)
		traceId = dbRecord.TraceId
		if err != nil {
			logger.Error("更新用户研报点击记录失败:%v", err)
			return
		}
		currTime := time.Now()
		dbRecord.ReadDurationSeconds = currTime.Sub(dbRecord.ClickTime).Milliseconds()
		err = userDao.UpdateSourceClicks(dbRecord)
	}
	if err != nil {
		logger.Error("插入用户研报点击记录失败:%v", err)
		return
	}
	_ = GetUserAndCountReadTimes(record.UserId)
	return
}

func CountMedia(record RecordCountDTO, mediaType string) (traceId string, err error) {
	sourceType := getSourceType(mediaType)
	if sourceType == "" {
		logger.Error("媒体类型错误,%s", mediaType)
		err = errors.New("illegal sourceType")
		return
	}
	record.SourceType = sourceType
	dao := convertUserToMediaFlow(record)
	if dao.TraceId == "" {
		traceId = generateTraceId(string(dao.SourceType))
		dao.TraceId = traceId
		err = userDao.CountSourceClicks(dao)
	} else {
		var dbRecord userDao.UserSourceClickFlow
		dbRecord, err = userDao.GetClickRecordByTraceId(dao.TraceId)
		traceId = dbRecord.TraceId
		if err != nil {
			logger.Error("更新用户研报点击记录失败:%v", err)
			return
		}
		currTime := time.Now()
		dbRecord.ReadDurationSeconds = currTime.Sub(dbRecord.ClickTime).Milliseconds()
		err = userDao.UpdateSourceClicks(dbRecord)
	}
	if err != nil {
		logger.Error("插入用户媒体点击记录失败:%v", err)
		return
	}
	_ = GetUserAndCountReadTimes(record.UserId)
	return
}

type HotReportDTO struct {
	ReportId int `gorm:"column:report_id"`
	Count    int `gorm:"column:count"`
}

func GetHotReports(begin string, end string, limit int) (dtoList []HotReportDTO) {
	counts, err := userDao.GetTimeDurationReportCountsById(begin, end, limit, userDao.ReportSourceType)
	if err != nil {
		logger.Error("获取最热研报数据失败:%v", err)
		return
	}
	if counts == nil {
		return []HotReportDTO{}
	}
	for _, count := range counts {
		dtoList = append(dtoList, convertUserToHotReportDTO(count))
	}
	return
}
func convertUserToReportFlow(record RecordCountDTO) userDao.UserSourceClickFlow {
	return userDao.UserSourceClickFlow{
		UserID:         record.UserId,
		Mobile:         record.Mobile,
		SourceId:       record.SourceId,
		TraceId:        record.TraceId,
		SourceType:     userDao.ReportSourceType,
		IPAddress:      record.IpAddress,
		Location:       record.Location,
		Referer:        record.Referer,
		AdditionalData: record.Additional,
	}
}
func convertUserToMediaFlow(media RecordCountDTO) userDao.UserSourceClickFlow {
	return userDao.UserSourceClickFlow{
		UserID:     media.UserId,
		TraceId:    media.TraceId,
		Mobile:     media.Mobile,
		SourceId:   media.SourceId,
		SourceType: media.SourceType,
	}
}
func convertUserToHotReportDTO(flow userDao.CountClickFlowById) HotReportDTO {
	return HotReportDTO{
		ReportId: flow.SourceId,
		Count:    flow.Count,
	}
}
